@@ -6,7 +6,6 @@
 # Distributed under the MIT license. See LICENSE for more info.
 
 import sys
-import time
 import socket
 import struct
 import fnmatch
@@ -22,31 +21,37 @@ Database = tango.Database
 Device = tango.gevent.DeviceProxy
 Attribute = tango.gevent.AttributeProxy
 
-ServerInfo = collections.namedtuple('ServerInfo',
-    ('name', 'type', 'instance', 'host', 'devices'))
-DeviceInfo = collections.namedtuple('DeviceInfo',
-    ('name', 'server', 'klass', 'alias', 'exported'))
-DatabaseInfo = collections.namedtuple('DatabaseInfo',
-    ('name', 'host', 'port', 'servers', 'devices', 'aliases'))
+ServerInfo = collections.namedtuple(
+    "ServerInfo", ("name", "type", "instance", "host", "devices")
+)
+DeviceInfo = collections.namedtuple(
+    "DeviceInfo", ("name", "server", "klass", "alias", "exported")
+)
+DatabaseInfo = collections.namedtuple(
+    "DatabaseInfo", ("name", "host", "port", "servers", "devices", "aliases")
+)
 
 
-log = logging.getLogger('tangoctl')
+log = logging.getLogger("tangoctl")
 
 
 def tango_error_str(exc_value, verbose=False, stack_message=-1):
     if verbose:
-        msg = '\n'.join(reversed(['{} @ {}: {}'.format(err.reason,
-                                                       err.origin,
-                                                       err.desc)
-                                  for err in exc_value]))
+        msg = "\n".join(
+            reversed(
+                [
+                    "{} @ {}: {}".format(err.reason, err.origin, err.desc)
+                    for err in exc_value
+                ]
+            )
+        )
     else:
         err = exc_value.args[stack_message]
-        msg = '{}: {}'.format(err.reason, err.desc)
+        msg = "{}: {}".format(err.reason, err.desc)
     return msg
 
 
 class ErrorHandler(object):
-
     def build_message(self, exc_type, exc_value, traceback):
         if issubclass(exc_type, tango.DevFailed):
             msg = tango_error_str(exc_value, verbose=self.verbose)
@@ -84,31 +89,38 @@ def get_db(db=None):
 
 
 def get_db_name(db):
-    return '{}:{}'.format(db.get_db_host(), db.get_db_port())
+    return "{}:{}".format(db.get_db_host(), db.get_db_port())
 
 
 def _get_server_devices(server_id, db=None):
     db = get_db(db)
     class_list = db.get_device_class_list(server_id)
-    return {name: DeviceInfo(name, server_id, klass, alias=None, exported=None)
-            for name, klass in zip(class_list[::2], class_list[1::2])}
+    return {
+        name: DeviceInfo(name, server_id, klass, alias=None, exported=None)
+        for name, klass in zip(class_list[::2], class_list[1::2])
+    }
 
 
 def _build_db_standard(db=None):
     db = get_db(db)
     all_servers, all_devices = {}, {}
     for server_id in db.get_server_list():
-        server_type, server_instance = server_id.split('/', 1)
+        server_type, server_instance = server_id.split("/", 1)
         devices = _get_server_devices(server_id, db=db)
         all_devices.update(devices)
-        device_names = { device_name for device_name in devices }
-        server = ServerInfo(server_id, server_type, server_instance, None,
-                            device_names)
+        device_names = {device_name for device_name in devices}
+        server = ServerInfo(server_id, server_type, server_instance, None, device_names)
         all_servers[server_id] = server
     host, port = db.get_db_host(), db.get_db_port_num()
-    name = '{}:{}'.format(host, port)
-    return DatabaseInfo(servers=all_servers, devices=all_devices, host=host,
-                        port=port, aliases=set(), name=name)
+    name = "{}:{}".format(host, port)
+    return DatabaseInfo(
+        servers=all_servers,
+        devices=all_devices,
+        host=host,
+        port=port,
+        aliases=set(),
+        name=name,
+    )
 
 
 def _build_db_quick(db_dev):
@@ -121,48 +133,54 @@ def _build_db_quick(db_dev):
     all_servers, all_devices, aliases = {}, {}, {}
     for row in range(row_nb):
         idx = row * column_nb
-        cells = data[idx:idx + column_nb]
+        cells = data[idx : idx + column_nb]
         dev_name, dev_alias, exported, host, server_id, klass = cells
         # handle garbage:
-        if not server_id or server_id.count('/') != 1:
+        if not server_id or server_id.count("/") != 1:
             continue
-        if not dev_name or dev_name.count('/') != 2:
-            print('skip', dev_name)
+        if not dev_name or dev_name.count("/") != 2:
+            print("skip", dev_name)
             continue
         if not dev_alias:
             dev_alias = None
         else:
             aliases[dev_alias] = dev_name
-        device = DeviceInfo(dev_name, server_id, klass, dev_alias,
-                            bool(int(exported)))
+        device = DeviceInfo(dev_name, server_id, klass, dev_alias, bool(int(exported)))
         server = all_servers.get(server_id)
         if server is None:
-            server_type, server_instance = server_id.split('/', 1)
-            server = ServerInfo(server_id, server_type, server_instance, host,
-                                set())
+            server_type, server_instance = server_id.split("/", 1)
+            server = ServerInfo(server_id, server_type, server_instance, host, set())
             all_servers[server_id] = server
         server.devices.add(dev_name)
         all_devices[dev_name.lower()] = device
     db = db_dev.get_device_db()
     host, port = db.get_db_host(), db.get_db_port_num()
-    name = '{}:{}'.format(host, port)
-    return DatabaseInfo(servers=all_servers, devices=all_devices,
-                        aliases=aliases, host=host, port=port, name=name)
+    name = "{}:{}".format(host, port)
+    return DatabaseInfo(
+        servers=all_servers,
+        devices=all_devices,
+        aliases=aliases,
+        host=host,
+        port=port,
+        name=name,
+    )
 
 
 def _build_db(db=None):
     db = get_db(db)
-    db_dev_name = '{}/{}'.format(get_db_name(db), db.dev_name())
+    db_dev_name = "{}/{}".format(get_db_name(db), db.dev_name())
     db_dev = Device(db_dev_name)
-    if hasattr(db_dev, 'DbMySqlSelect'):
+    if hasattr(db_dev, "DbMySqlSelect"):
         return _build_db_quick(db_dev)
     else:
         return _build_db_standard(db=db)
 
+
 # TODO: consider creating a timestamped ~/.cache/tangoctl/db_name.pkl with db_info
 
 DB_INFO = {}
 
+
 def get_db_info(db=None):
     global DB_INFO
     db = get_db(db)
@@ -174,126 +192,154 @@ def get_db_info(db=None):
 
 
 IOR = collections.namedtuple(
-    'IOR',
-    'first dtype_length dtype nb_profile tag '
-    'length major minor wtf host_length ip port body')
+    "IOR",
+    "first dtype_length dtype nb_profile tag "
+    "length major minor wtf host_length ip port body",
+)
 
 
 def _ascii_to_bytes(s):
     convert = lambda x: struct.Struct(">B").pack(int(x, 16))
-    return b''.join(convert(s[i:i + 2]) for i in range(0, len(s), 2))
+    return b"".join(convert(s[i : i + 2]) for i in range(0, len(s), 2))
 
 
 def _parse_ior(encoded_ior):
-    assert encoded_ior[:4] == 'IOR:'
+    assert encoded_ior[:4] == "IOR:"
     ior = _ascii_to_bytes(encoded_ior[4:])
-    dtype_length = struct.unpack_from('II', ior)[-1]
-    form = 'II{:d}sIIIBBHI'.format(dtype_length)
+    dtype_length = struct.unpack_from("II", ior)[-1]
+    form = "II{:d}sIIIBBHI".format(dtype_length)
     host_length = struct.unpack_from(form, ior)[-1]
-    form = 'II{:d}sIIIBBHI{:d}sH0I'.format(dtype_length, host_length)
+    form = "II{:d}sIIIBBHI{:d}sH0I".format(dtype_length, host_length)
     values = struct.unpack_from(form, ior)
-    values += (ior[struct.calcsize(form):],)
+    values += (ior[struct.calcsize(form) :],)
     strip = lambda x: x[:-1] if isinstance(x, bytes) else x
     return IOR(*map(strip, values))
 
 
 TYPE_MAP = {
-    tango.CmdArgType.DevVoid: 'None',
-    tango.CmdArgType.DevState: 'state',
-    tango.CmdArgType.DevEnum: 'enum',
-    tango.CmdArgType.DevEncoded: '(str, bytes)',
-    tango.CmdArgType.DevBoolean: 'bool',
-    tango.CmdArgType.DevUChar: 'bytes',
-    tango.CmdArgType.DevString: 'str',
-    tango.CmdArgType.DevShort: 'int16',
-    tango.CmdArgType.DevUShort: 'uint16',
-    tango.CmdArgType.DevLong: 'int32',
-    tango.CmdArgType.DevLong64: 'int64',
-    tango.CmdArgType.DevULong: 'uint32',
-    tango.CmdArgType.DevULong64: 'uint64',
-    tango.CmdArgType.DevFloat: 'float32',
-    tango.CmdArgType.DevDouble: 'float64',
+    tango.CmdArgType.DevVoid: "void",
+    tango.CmdArgType.DevState: "state",
+    tango.CmdArgType.DevEnum: "enum",
+    tango.CmdArgType.DevEncoded: "(str, bytes)",
+    tango.CmdArgType.DevBoolean: "bool",
+    tango.CmdArgType.DevUChar: "bytes",
+    tango.CmdArgType.DevString: "str",
+    tango.CmdArgType.DevShort: "int16",
+    tango.CmdArgType.DevUShort: "uint16",
+    tango.CmdArgType.DevLong: "int32",
+    tango.CmdArgType.DevLong64: "int64",
+    tango.CmdArgType.DevULong: "uint32",
+    tango.CmdArgType.DevULong64: "uint64",
+    tango.CmdArgType.DevFloat: "float32",
+    tango.CmdArgType.DevDouble: "float64",
+    tango.CmdArgType.DevVarStringArray: "[str]",
+    tango.CmdArgType.DevVarBooleanArray: "[bool]",
+    tango.CmdArgType.DevVarCharArray: "[bytes]",
+    tango.CmdArgType.DevVarDoubleArray: "[float64]",
+    tango.CmdArgType.DevVarDoubleStringArray: "[float64, str]",
+    tango.CmdArgType.DevVarFloatArray: "[float32]",
+    tango.CmdArgType.DevVarLong64Array: "[int64]",
+    tango.CmdArgType.DevVarLongArray: "[int32]",
+    tango.CmdArgType.DevVarLongStringArray: "[int32, str]",
+    tango.CmdArgType.DevVarShortArray: "[int16]",
+    tango.CmdArgType.DevVarStateArray: "[state]",
+    tango.CmdArgType.DevVarULong64Array: "[uint64]",
+    tango.CmdArgType.DevVarULongArray: "[uint32]",
+    tango.CmdArgType.DevVarUShortArray: "[uint16]",
 }
 
+
 def type_str(t, format=None):
     if isinstance(t, int):
         t = tango.CmdArgType.values[t]
     tstr = TYPE_MAP[t]
     if format == tango.AttrDataFormat.SPECTRUM:
-        tstr = '[{}]'.format(tstr)
-    elif  format == tango.AttrDataFormat.IMAGE:
-        tstr = '[[{}]]'.format(tstr)
+        tstr = "[{}]".format(tstr)
+    elif format == tango.AttrDataFormat.IMAGE:
+        tstr = "[[{}]]".format(tstr)
     return tstr
 
 
-NULL = ('Not specified', 'No description',
-        'No unit', 'No standard unit', 'No display unit')
+NULL = (
+    "Not specified",
+    "No description",
+    "No unit",
+    "No standard unit",
+    "No display unit",
+)
 
 
 def value_str(v):
-    return '---' if v in NULL else str(v)
+    return "---" if v in NULL else str(v)
 
 
 def attr_value_str_color(v):
-    if v['has_failed']:
-        err = v['error'][0]
-        value_str = '{}: {}'.format(err.reason, err.desc)
-        fg = 'bright_red'
+    if v["has_failed"]:
+        err = v["error"][0]
+        value_str = "{}: {}".format(err.reason, err.desc)
+        fg = "bright_red"
         return value_str, fg
-    quality = v['quality']
+    quality = v["quality"]
     if quality == tango.AttrQuality.ATTR_INVALID:
-        value_str = '---'
+        value_str = "---"
     else:
-        value_str = str(v['value'])
-    unit = v['unit']
-    unit = '' if unit == 'No unit' else ' ' + unit
+        value_str = str(v["value"])
+    unit = v["unit"]
+    unit = "" if unit == "No unit" else " " + unit
     fg = None
     if quality == tango.AttrQuality.ATTR_INVALID:
-        fg = 'bright_red'
+        fg = "bright_red"
     elif quality == tango.AttrQuality.ATTR_WARNING:
-        fg = 'yellow'
+        fg = "yellow"
     elif quality == tango.AttrQuality.ATTR_ALARM:
-        fg = 'red'
+        fg = "red"
     elif quality == tango.AttrQuality.ATTR_CHANGING:
-        fg = 'bright_blue'
-    return '{}{}'.format(value_str, unit), fg
+        fg = "bright_blue"
+    return "{}{}".format(value_str, unit), fg
 
 
 def access_str(access):
     if access == tango.AttrWriteType.READ:
-        return 'R'
+        return "R"
     elif access == tango.AttrWriteType.WRITE:
-        return 'W'
+        return "W"
     elif access == tango.AttrWriteType.READ_WRITE:
-        return 'RW'
+        return "RW"
     elif access == tango.AttrWriteType.READ_WITH_WRITE:
-        return 'RWW'
-    return '?'
+        return "RWW"
+    return "?"
+
 
 # ----- API -----
 
-def iter_devices(device=None, server=None, klass=None, host=None,
-                 exclude_dserver=True, db=None):
+
+def iter_devices(
+    device=None, server=None, klass=None, host=None, exclude_dserver=True, db=None
+):
     db = get_db(db)
     db_info = get_db_info(db=db)
 
     devs, servers = db_info.devices, db_info.servers
     devices = (devs[dname] for dname in sorted(devs))
     if exclude_dserver:
-        devices = (d for d in devices if d.klass != 'DServer')
+        devices = (d for d in devices if d.klass != "DServer")
     if device:
         device = device.lower()
-        devices = (d for d in devices
-                   if fnmatch.fnmatch(d.name.lower(), device) or \
-                     (d.alias and fnmatch.fnmatch(d.alias.lower(), device)))
+        devices = (
+            d
+            for d in devices
+            if fnmatch.fnmatch(d.name.lower(), device)
+            or (d.alias and fnmatch.fnmatch(d.alias.lower(), device))
+        )
     if klass:
         devices = (d for d in devices if fnmatch.fnmatch(d.klass, klass))
     if server:
         devices = (d for d in devices if fnmatch.fnmatch(d.server, server))
     if host:
         host = host.lower()
-        devices = (d for d in devices
-                   if fnmatch.fnmatch(servers[d.server].host.lower(), host))
+        devices = (
+            d for d in devices if fnmatch.fnmatch(servers[d.server].host.lower(), host)
+        )
     return devices
 
 
@@ -311,10 +357,10 @@ def iter_servers(server=None, server_type=None, server_instance=None, db=None):
 
 
 def server_info(name, db=None):
-    dserver = 'dserver/' + name
+    dserver = "dserver/" + name
     r = device_info(dserver, db=db)
-    r.pop('class')
-    r['name'] = r.pop('server')
+    r.pop("class")
+    r["name"] = r.pop("server")
     return r
 
 
@@ -324,7 +370,7 @@ def expr_to_iservers(expr, db=None):
     if expr is None:
         return servers
     if not isinstance(expr, (list, tuple)):
-        expr = expr,
+        expr = (expr,)
     f = lambda server: any(fnmatch.fnmatch(server, pat) for pat in expr)
     return filter(f, servers)
 
@@ -332,12 +378,12 @@ def expr_to_iservers(expr, db=None):
 def server_ping(server_names):
     def ping(server):
         try:
-            r = Device('dserver/' + server).ping()
+            r = Device("dserver/" + server).ping()
         except Exception as err:
             r = err
         return server, r
-    ping_tasks = [gevent.spawn(ping, server_name)
-                  for server_name in server_names]
+
+    ping_tasks = [gevent.spawn(ping, server_name) for server_name in server_names]
     for result in gevent.iwait(ping_tasks):
         yield result.get()
 
@@ -345,12 +391,12 @@ def server_ping(server_names):
 def server_stop(server_names):
     def stop(server):
         try:
-            r = Device('dserver/' + server).Kill()
+            r = Device("dserver/" + server).Kill()
         except Exception as err:
             r = err
         return server, r
-    stop_tasks = [gevent.spawn(stop, server_name)
-                  for server_name in server_names]
+
+    stop_tasks = [gevent.spawn(stop, server_name) for server_name in server_names]
     for result in gevent.iwait(stop_tasks):
         yield result.get()
 
@@ -363,7 +409,7 @@ def server_add(server, devices, db=None):
         dev_info._class = dev_class
         dev_info.server = server
         db.add_device(dev_info)
-        yield server, '{}:{}'.format(dev_class, dev_name)
+        yield server, "{}:{}".format(dev_class, dev_name)
 
 
 def server_delete(server, db=None):
@@ -387,7 +433,7 @@ def expr_to_idevices(expr, exclude_dserver=True, db=None):
     devs = db_info.devices
     devices = (devs[dname] for dname in sorted(devs))
     if exclude_dserver:
-        devices = (d for d in devices if d.klass != 'DServer')
+        devices = (d for d in devices if d.klass != "DServer")
     if expr is None:
         return devices
     expr_lower = [pat.lower() for pat in expr]
@@ -402,8 +448,8 @@ def device_ping(device_names, exclude_dserver=True):
         except Exception as err:
             r = err
         return device_name, r
-    ping_tasks = [gevent.spawn(ping, device_name)
-                  for device_name in device_names]
+
+    ping_tasks = [gevent.spawn(ping, device_name) for device_name in device_names]
     for result in gevent.iwait(ping_tasks):
         yield result.get()
 
@@ -415,12 +461,20 @@ def device_info(name, db=None):
     try:
         host = socket.gethostbyaddr(ior.ip)[0]
     except:
-        host = ''
-    r = {'class':info.class_name, 'server':info.ds_full_name,
-         'exported':info.exported,'name':info.name, 'pid':info.pid,
-         'last started':info.started_date, 'last stopped':info.stopped_date,
-         'version': info.version, 'ip': ior.ip.decode(), 'port': ior.port,
-         'host': host}
+        host = ""
+    r = {
+        "class": info.class_name,
+        "server": info.ds_full_name,
+        "exported": info.exported,
+        "name": info.name,
+        "pid": info.pid,
+        "last started": info.started_date,
+        "last stopped": info.stopped_date,
+        "version": info.version,
+        "ip": ior.ip.decode(),
+        "port": ior.port,
+        "host": host,
+    }
     return r
 
 
@@ -440,11 +494,11 @@ def device_attribute_read(name, attribute):
 def _attr_to_dict(config=None, value=None):
     result = {}
     if value:
-        result.update({k:getattr(value, k) for k in dir(value)})
+        result.update({k: getattr(value, k) for k in dir(value)})
         if value.has_failed:
-            result['error'] = value.get_err_stack()
+            result["error"] = value.get_err_stack()
     if config:
-        result.update({k:getattr(config, k) for k in dir(config)})
+        result.update({k: getattr(config, k) for k in dir(config)})
     return result
 
 
@@ -452,7 +506,7 @@ def expr_to_iattrs(expr, attribute_names):
     if expr is None:
         return attribute_names
     if not isinstance(expr, (list, tuple)):
-        expr = expr,
+        expr = (expr,)
     expr_lower = [pat.lower() for pat in expr]
     f = lambda name: any(fnmatch.fnmatch(name.lower(), pat) for pat in expr_lower)
     return filter(f, attribute_names)
@@ -464,13 +518,17 @@ def attributes_read(dev_attr_map):
             d = Device(device_name)
             attrs = tuple(expr_to_iattrs(attrs, d.get_attribute_list()))
             values = {value.name: value for value in d.read_attributes(attrs)}
-            r = [_attr_to_dict(config, values[config.name])
-                 for config in d.get_attribute_config_ex(attrs)]
+            r = [
+                _attr_to_dict(config, values[config.name])
+                for config in d.get_attribute_config_ex(attrs)
+            ]
         except Exception as err:
             r = err
         return device_name, r
-    read_tasks = [gevent.spawn(read, name, attrs)
-                  for name, attrs in dev_attr_map.items()]
+
+    read_tasks = [
+        gevent.spawn(read, name, attrs) for name, attrs in dev_attr_map.items()
+    ]
     for result in gevent.iwait(read_tasks):
         yield result.get()
 
@@ -493,7 +551,10 @@ def device_attributes(dev_name, attribute=None):
     attrs = device.attribute_list_query_ex()
     if attribute:
         attribute = attribute.lower()
-        def filt(attr): fnmatch.fnmatch(attr.name.lower(), attribute)
+
+        def filt(attr):
+            fnmatch.fnmatch(attr.name.lower(), attribute)
+
         attrs = filter(filt, attrs)
     return sorted(attrs, key=lambda a: a.name)
 
@@ -503,7 +564,10 @@ def device_commands(dev_name, command=None):
     cmds = device.command_list_query()
     if command:
         command = command.lower()
-        def filt(cmd): fnmatch.fnmatch(cmd.cmd_name.lower(), command)
+
+        def filt(cmd):
+            fnmatch.fnmatch(cmd.cmd_name.lower(), command)
+
         attrs = filter(filt, cmds)
     return sorted(cmds, key=lambda a: a.cmd_name)
 
@@ -517,7 +581,7 @@ def expr_to_iprops(expr, prop_names):
     if expr is None:
         return prop_names
     if not isinstance(expr, (list, tuple)):
-        expr = expr,
+        expr = (expr,)
     expr_lower = [pat.lower() for pat in expr]
     f = lambda name: any(fnmatch.fnmatch(name.lower(), pat) for pat in expr_lower)
     return filter(f, prop_names)
@@ -525,45 +589,44 @@ def expr_to_iprops(expr, prop_names):
 
 def device_properties_read(dev_name, properties, db=None):
     db = get_db(db)
-    props = tuple(db.get_device_property_list(dev_name, '*'))
+    props = tuple(db.get_device_property_list(dev_name, "*"))
     props = tuple(expr_to_iprops(properties, props))
     return db.get_device_property(dev_name, props)
 
 
 def device_property_write(dev_name, property, value, db=None):
     db = get_db(db)
-    return db.put_device_property(dev_name, {str(property):str(value)})
+    return db.put_device_property(dev_name, {str(property): str(value)})
 
 
 def device_property_list(dev_name, db=None):
     db = get_db(db)
-    return tuple(db.get_device_property_list(dev_name, '*'))
+    return tuple(db.get_device_property_list(dev_name, "*"))
 
 
 def starter_names(db=None):
     db = get_db(db)
-    return [s.split('/', 1)[-1] for s in db.get_server_list('Starter/*')]
+    return [s.split("/", 1)[-1] for s in db.get_server_list("Starter/*")]
 
 
 def starter_device_name(host, db=None):
     db = get_db(db)
-    dev_class_name = db.get_device_class_list('Starter/{}'.format(host))
+    dev_class_name = db.get_device_class_list("Starter/{}".format(host))
     for d, k in zip(dev_class_name[::2], dev_class_name[1::2]):
-        if k == 'Starter':
+        if k == "Starter":
             return d
 
 
 def human_starter_server_state(state):
-    if state == 'FAULT':
-        return 'NOT running'
-    elif state == 'ON':
-        return 'Running'
-    elif state == 'MOVING':
-        return 'Starting/Stopping'
+    if state == "FAULT":
+        return "NOT running"
+    elif state == "ON":
+        return "Running"
+    elif state == "MOVING":
+        return "Starting/Stopping"
 
 
 class Starter:
-
     def __init__(self, name, db=None):
         self._log = log.getChild(name)
         self.name = name
@@ -582,31 +645,36 @@ class Starter:
 
     @property
     def running_servers(self):
-        return self._filtered_servers(lambda s: s['state'] == 'ON')
+        return self._filtered_servers(lambda s: s["state"] == "ON")
 
     @property
     def transition_servers(self):
-        return self._filtered_servers(lambda s: s['state'] == 'MOVING')
+        return self._filtered_servers(lambda s: s["state"] == "MOVING")
 
     @property
     def stopped_servers(self):
-        return self._filtered_servers(lambda s: s['state'] == 'FAULT')
+        return self._filtered_servers(lambda s: s["state"] == "FAULT")
 
     @property
     def controlled_servers(self):
-        return self._filtered_servers(lambda s: s['controlled'])
+        return self._filtered_servers(lambda s: s["controlled"])
 
     @property
     def servers(self):
         db_info = get_db_info(db=self.db)
         db_servers = db_info.servers
         servers = {}
-        starter_servers = self.starter_device['servers'].value or []
+        starter_servers = self.starter_device["servers"].value or []
         for server in starter_servers:
             name, state, controlled, level, *_ = server.split()
             level = int(level)
-            servers[name] = dict(name=name, state=state, controlled=controlled,
-                                 level=level, info=db_servers[name])
+            servers[name] = dict(
+                name=name,
+                state=state,
+                controlled=controlled,
+                level=level,
+                info=db_servers[name],
+            )
         return servers
 
     def __getitem__(self, name):
@@ -617,73 +685,78 @@ class Starter:
 
     def wait_for_state(self, name, cond):
         while True:
-            state = self[name]['state']
+            state = self[name]["state"]
             if cond(state):
                 return state
             gevent.sleep(0.2)
 
     def kill_server(self, name, dry_run=False):
-        state = self[name]['state']
-        if state == 'FAULT':
-            self._log.info('skipped killing %r (%s)',
-                           name, human_starter_server_state(state))
+        state = self[name]["state"]
+        if state == "FAULT":
+            self._log.info(
+                "skipped killing %r (%s)", name, human_starter_server_state(state)
+            )
             return True
-        self._log.info('killing %r', name)
+        self._log.info("killing %r", name)
         if not dry_run:
             try:
                 self.starter_device.HardKillServer(name)
             except tango.DevFailed as err:
                 err = err.args[0].desc
-                state = self[name]['state']
-                if state == 'FAULT':
+                state = self[name]["state"]
+                if state == "FAULT":
                     self._log.warning(
-                        'server %r was killed but reported: %s', name, err)
+                        "server %r was killed but reported: %s", name, err
+                    )
                     return True
                 else:
-                    self._log.error('failed to kill %r: %s', name, err)
+                    self._log.error("failed to kill %r: %s", name, err)
                     return False
-            state = self.wait_for_state(
-                name, lambda s: s in ('MOVING', 'FAULT'))
+            state = self.wait_for_state(name, lambda s: s in ("MOVING", "FAULT"))
             if not state:
-                self._log.warn('failed to wait to kill %r', name)
+                self._log.warn("failed to wait to kill %r", name)
                 return False
-            if state == 'FAULT':
-                self._log.info('killed %r', name)
+            if state == "FAULT":
+                self._log.info("killed %r", name)
                 return True
-            state = self.wait_for_state(name, lambda s: s == 'FAULT')
+            state = self.wait_for_state(name, lambda s: s == "FAULT")
             if not state:
-                self._log.warn('failed to kill %r', name)
+                self._log.warn("failed to kill %r", name)
                 return False
-        self._log.info('killed %r', name)
+        self._log.info("killed %r", name)
         return True
 
     def stop_server(self, name, dry_run=False):
         server = self[name]
-        state, level = server['state'], server['level']
-        if state == 'FAULT':
-            self._log.info('skipped stopping %r (%s) (level %d)',
-                           name, human_starter_server_state(state), level)
+        state, level = server["state"], server["level"]
+        if state == "FAULT":
+            self._log.info(
+                "skipped stopping %r (%s) (level %d)",
+                name,
+                human_starter_server_state(state),
+                level,
+            )
             return True
-        self._log.info('stopping %r (level %d)', name, level)
+        self._log.info("stopping %r (level %d)", name, level)
         if not dry_run:
             try:
                 self.starter_device.DevStop(name)
             except tango.DevFailed as err:
                 err = err.args[0].desc
-                self._log.warning('failed to stop on %r (%r)', name, err)
+                self._log.warning("failed to stop on %r (%r)", name, err)
                 return False
-            state = self.wait_for_state(name, lambda s: s in ('MOVING', 'FAULT'))
+            state = self.wait_for_state(name, lambda s: s in ("MOVING", "FAULT"))
             if not state:
-                self._log.warn('failed to wait to stop %r', name)
+                self._log.warn("failed to wait to stop %r", name)
                 return False
-            if state == 'FAULT':
-                self._log.info('stopped %r', name)
+            if state == "FAULT":
+                self._log.info("stopped %r", name)
                 return True
-            state = self.wait_for_state(name, lambda s: s == 'FAULT')
+            state = self.wait_for_state(name, lambda s: s == "FAULT")
             if not state:
-                self._log.warn('failed to stop %r', name)
+                self._log.warn("failed to stop %r", name)
                 return False
-        self._log.info('stopped %r', name)
+        self._log.info("stopped %r", name)
         return True
 
     def stop_kill_server(self, name, timeout=5, dry_run=False):
@@ -692,34 +765,34 @@ class Starter:
         try:
             stopped = task.get(timeout=timeout)
         except gevent.Timeout:
-            self._log.warning(
-                'timeout trying to stop %s. Performing a kill', name)
+            self._log.warning("timeout trying to stop %s. Performing a kill", name)
             stopped = False
             task.kill()
         if not stopped:
             self.kill_server(name, dry_run=dry_run)
 
     def start_server(self, name, dry_run=False):
-        state = self[name]['state']
-        if state == 'ON':
-            self._log.info('skipped starting %r (%s)',
-                           name, human_starter_server_state(state))
+        state = self[name]["state"]
+        if state == "ON":
+            self._log.info(
+                "skipped starting %r (%s)", name, human_starter_server_state(state)
+            )
             return True
-        self._log.info('starting %r', name)
+        self._log.info("starting %r", name)
         if not dry_run:
             self.starter_device.DevStart(name)
-            state = self.wait_for_state(name, lambda s: s in ('MOVING', 'ON'))
+            state = self.wait_for_state(name, lambda s: s in ("MOVING", "ON"))
             if not state:
-                self._log.warn('failed to start %r', name)
+                self._log.warn("failed to start %r", name)
                 return False
-            if state == 'ON':
-                self._log.info('started %r', name)
+            if state == "ON":
+                self._log.info("started %r", name)
                 return True
-            state = self.wait_for_state(name, lambda s: s == 'ON')
+            state = self.wait_for_state(name, lambda s: s == "ON")
             if not state:
-                self._log.warn('failed to start %r', name)
+                self._log.warn("failed to start %r", name)
                 return False
-        self._log.info('started %r', name)
+        self._log.info("started %r", name)
         return True
 
     def restart_server(self, name):
@@ -746,17 +819,17 @@ def iter_starters(starter=None, db=None):
     return result
 
 
-def iter_starter_servers(servers, server_name=None, server_type=None,
-                         server_instance=None):
+def iter_starter_servers(
+    servers, server_name=None, server_type=None, server_instance=None
+):
     if server_name:
-        servers = (s for s in servers
-                   if fnmatch.fnmatch(s['info'].name, server_name))
+        servers = (s for s in servers if fnmatch.fnmatch(s["info"].name, server_name))
     if server_type:
-        servers = (s for s in servers
-                   if fnmatch.fnmatch(s['info'].type, server_type))
+        servers = (s for s in servers if fnmatch.fnmatch(s["info"].type, server_type))
     if server_instance:
-        servers = (s for s in servers
-                   if fnmatch.fnmatch(s['info'].instance, server_instance))
+        servers = (
+            s for s in servers if fnmatch.fnmatch(s["info"].instance, server_instance)
+        )
     return servers
 
 
@@ -782,36 +855,53 @@ def find_starters_for_servers(names, db=None):
 
 
 def find_starters_for_filtered_servers(
-        starter_name=None, server_name=None, server_type=None,
-        server_instance=None, filter_func=None, db=None):
+    starter_name=None,
+    server_name=None,
+    server_type=None,
+    server_instance=None,
+    filter_func=None,
+    db=None,
+):
     starters_seq = iter_starters(starter_name, db=db)
     result = {}
     for starter, servers in starters_servers(*starters_seq).items():
         servers = servers.values() if servers else ()
-        servers = iter_starter_servers(servers, server_name=server_name,
-                                       server_type=server_type,
-                                       server_instance=server_instance)
+        servers = iter_starter_servers(
+            servers,
+            server_name=server_name,
+            server_type=server_type,
+            server_instance=server_instance,
+        )
         if filter_func:
             servers = filter(filter_func, servers)
         servers = list(servers)
         if servers:
             result[starter] = servers
     return result
-    
+
 
 def find_starters_levels_for_filtered_servers(
-        starter_name=None, server_name=None, server_type=None,
-        server_instance=None, filter_func=None, db=None):
+    starter_name=None,
+    server_name=None,
+    server_type=None,
+    server_instance=None,
+    filter_func=None,
+    db=None,
+):
 
     starters_map = find_starters_for_filtered_servers(
-        starter_name=starter_name, server_name=server_name,
-        server_type=server_type, server_instance=server_instance,
-        filter_func=filter_func, db=db)
-    #group by levels
-    levels = collections.defaultdict(lambda : collections.defaultdict(list))
+        starter_name=starter_name,
+        server_name=server_name,
+        server_type=server_type,
+        server_instance=server_instance,
+        filter_func=filter_func,
+        db=db,
+    )
+    # group by levels
+    levels = collections.defaultdict(lambda: collections.defaultdict(list))
     for starter, servers in starters_map.items():
         for server in servers:
-            level_starters = levels[server['level']]
+            level_starters = levels[server["level"]]
             level_starters[starter].append(server)
     return levels
 
@@ -839,66 +929,100 @@ def stop_server(name, db=None):
 
 
 def starter_stop_servers(
-        starter_name=None, server_name=None, server_type=None,
-        server_instance=None, timeout=None, stop_timeout=5,
-        dry_run=False, db=None):
+    starter_name=None,
+    server_name=None,
+    server_type=None,
+    server_instance=None,
+    timeout=None,
+    stop_timeout=5,
+    dry_run=False,
+    db=None,
+):
 
     with gevent.Timeout(timeout):
         levels = find_starters_levels_for_filtered_servers(
-            starter_name=starter_name, server_name=server_name,
-            server_type=server_type, server_instance=server_instance,
-            filter_func=lambda s: s['level'] > 0,
-            db=db)
+            starter_name=starter_name,
+            server_name=server_name,
+            server_type=server_type,
+            server_instance=server_instance,
+            filter_func=lambda s: s["level"] > 0,
+            db=db,
+        )
         for level in sorted(levels, reverse=True):
             level_tasks = []
             for starter, servers in levels[level].items():
                 stop = starter.stop_kill_server
-                tasks = [gevent.spawn(stop, server['name'],
-                                      timeout=stop_timeout, dry_run=dry_run)
-                         for server in servers]
+                tasks = [
+                    gevent.spawn(
+                        stop, server["name"], timeout=stop_timeout, dry_run=dry_run
+                    )
+                    for server in servers
+                ]
                 level_tasks.extend(tasks)
-            log.info('beginning to stop servers at level {}'.format(level))
+            log.info("beginning to stop servers at level {}".format(level))
             gevent.joinall(level_tasks)
-            log.info('finished stopping level {}'.format(level))
+            log.info("finished stopping level {}".format(level))
 
 
 def starter_start_servers(
-        starter_name=None, server_name=None, server_type=None,
-        server_instance=None, timeout=None, dry_run=False, db=None):
+    starter_name=None,
+    server_name=None,
+    server_type=None,
+    server_instance=None,
+    timeout=None,
+    dry_run=False,
+    db=None,
+):
 
     with gevent.Timeout(timeout):
         levels = find_starters_levels_for_filtered_servers(
-            starter_name=starter_name, server_name=server_name,
-            server_type=server_type, server_instance=server_instance,
-            filter_func=lambda s: s['level'] > 0,
-            db=db)
+            starter_name=starter_name,
+            server_name=server_name,
+            server_type=server_type,
+            server_instance=server_instance,
+            filter_func=lambda s: s["level"] > 0,
+            db=db,
+        )
         for level in sorted(levels):
             level_tasks = []
             for starter, servers in levels[level].items():
                 stop = starter.start_server
-                tasks = [gevent.spawn(stop, server['name'], dry_run=dry_run)
-                         for server in servers]
+                tasks = [
+                    gevent.spawn(stop, server["name"], dry_run=dry_run)
+                    for server in servers
+                ]
                 level_tasks.extend(tasks)
-            log.info('beginning to start servers at level {}'.format(level))
+            log.info("beginning to start servers at level {}".format(level))
             gevent.joinall(level_tasks)
-            log.info('finished starting servers at level {}'.format(level))
+            log.info("finished starting servers at level {}".format(level))
 
 
 def starter_restart_servers(
-        starter_name=None, server_name=None, server_type=None,
-        server_instance=None, timeout=None, dry_run=False, db=None):
+    starter_name=None,
+    server_name=None,
+    server_type=None,
+    server_instance=None,
+    timeout=None,
+    dry_run=False,
+    db=None,
+):
 
     db = get_db(db)
 
     with gevent.Timeout(timeout):
         starter_stop_servers(
-            starter_name=starter_name, server_name=server_name,
-            server_type=server_type, server_instance=server_instance,
-            dry_run=dry_run, db=db)
-        starter_stop_servers(
-            starter_name=starter_name, server_name=server_name,
-            server_type=server_type, server_instance=server_instance,
-            dry_run=dry_run, db=db)
-
-
-
+            starter_name=starter_name,
+            server_name=server_name,
+            server_type=server_type,
+            server_instance=server_instance,
+            dry_run=dry_run,
+            db=db,
+        )
+        starter_start_servers(
+            starter_name=starter_name,
+            server_name=server_name,
+            server_type=server_type,
+            server_instance=server_instance,
+            dry_run=dry_run,
+            db=db,
+        )
